SystemOrganization addCategory: #SoftwareDesign!
SystemOrganization addCategory: #'SoftwareDesign-Model'!

CircleMorph subclass: #SDNavNode
	instanceVariableNames: 'gridPosition name edges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SoftwareDesign-Model'!

!SDNavNode class methodsFor: 'instance creation' stamp: 'GNS 11/18/2012 15:41'!
createWithName: aName

	| node |
	
	node := SDNavNode new.
	node name: aName.		
	
	^ node ! !

!SDNavNode class methodsFor: 'instance creation' stamp: 'GNS 11/18/2012 16:59'!
createWithName: aName on: aPoint

	| node |
	
	node := SDNavNode new.
	node name: aName.	
	node gridPosition: aPoint.	
	
	^ node ! !

!SDNavNode methodsFor: 'accessing' stamp: 'GNS 11/20/2012 16:41'!
addConnectionTo: aNode withWeight: aWeight

	| edge |
	
	edge := SDNavEdge newWithStartNode: self andEndNode: aNode withWeight: aWeight.
	
	self edges add: edge.
	
	^ edge! !

!SDNavNode methodsFor: 'utilities' stamp: 'GNS 11/18/2012 13:45'!
distanceTo: aNode

	
	self edges do: [ :edge |
		(edge to = aNode) ifTrue: [^ edge weights ]
		].
	^ 1000! !

!SDNavNode methodsFor: 'drawing' stamp: 'GNS 11/20/2012 16:48'!
drawYourselfOn: aCanvas

	| label |
	
	aCanvas addMorph: self. 
	self
		extent: 30@30; 
		center: aCanvas topLeft + (self gridPosition * 50).
		
	label := StringMorph contents: self name.
	self addMorph: label.
	label
		fontName: Preferences standardDefaultTextFont familyName size: 8;
		lock;
		center: self center.! !

!SDNavNode methodsFor: 'accessing' stamp: 'GNS 11/18/2012 12:26'!
edges
	"Answer the value of edges"
	
	edges ifNil: [
		edges := OrderedCollection new.].
	
	^ edges! !

!SDNavNode methodsFor: 'accessing' stamp: 'GNS 11/18/2012 16:59'!
edges: anObject
	"Set the value of edges"

	edges := anObject! !

!SDNavNode methodsFor: 'accessing' stamp: 'GNS 11/18/2012 17:13'!
gridPosition
	"Answer the value of gridposition"

	^ gridPosition! !

!SDNavNode methodsFor: 'accessing' stamp: 'GNS 11/18/2012 17:13'!
gridPosition: anObject
	"Set the value of gridposition"

	gridPosition := anObject! !

!SDNavNode methodsFor: 'accessing' stamp: 'GNS 11/20/2012 16:18'!
name

	^ name! !

!SDNavNode methodsFor: 'accessing' stamp: 'GNS 11/18/2012 11:54'!
name: anObject
	"Set the value of name"

	name := anObject! !

Object subclass: #SDNavDijkstra
	instanceVariableNames: 'name count edges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SoftwareDesign-Model'!

Object subclass: #SDNavDijkstraAlgorithm
	instanceVariableNames: 'startNode graph distances visitedNodes unvisitedNodes predecessors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SoftwareDesign-Model'!

!SDNavDijkstraAlgorithm methodsFor: 'accessing' stamp: 'GNS 11/18/2012 12:56'!
distances
	"Answer the value of distance"
	
	distances ifNil: [
		distances := Dictionary new.].
	
	^ distances! !

!SDNavDijkstraAlgorithm methodsFor: 'as yet unclassified' stamp: 'GNS 11/18/2012 16:21'!
execute: aStartNode

	self distances at: aStartNode put: 0.
	self unvisitedNodes add: aStartNode.
	
	[self unvisitedNodes size > 0] whileTrue: [ | aNode |
		
		aNode := self getMininumOfUnVisitedNodes.
		self visitedNodes add: aNode.
		self unvisitedNodes remove: aNode.
		self findMinimalDistances: aNode.].! !

!SDNavDijkstraAlgorithm methodsFor: 'as yet unclassified' stamp: 'GNS 11/18/2012 16:18'!
findMinimalDistances: aNode

	| unvisitedNeighbors |
	
	unvisitedNeighbors := self unvisitedNeighborsOf: aNode.
	unvisitedNeighbors do: [ :unvisitedNeighbor |
		
		| tempDistance |
		tempDistance := (self shortestDistanceOf: aNode) + (aNode distanceTo: unvisitedNeighbor).
		
		((self shortestDistanceOf: unvisitedNeighbor) > tempDistance ) ifTrue: [
			self distances at: unvisitedNeighbor put: tempDistance.
			self predecessors at: unvisitedNeighbor put: aNode.
			self unvisitedNodes add: unvisitedNeighbor.].
	].! !

!SDNavDijkstraAlgorithm methodsFor: 'accessing' stamp: 'GNS 11/18/2012 16:17'!
getMininumOfUnVisitedNodes
	
	| minimum |
	
	minimum := nil.
	
	self unvisitedNodes do: [ :unvisitedNode |
		minimum
			ifNil: [minimum := unvisitedNode.]
			ifNotNil: [
				((self shortestDistanceOf: unvisitedNode)
					< (self shortestDistanceOf: minimum)) ifTrue: [minimum := unvisitedNode.]].	].
	^ minimum! !

!SDNavDijkstraAlgorithm methodsFor: 'accessing' stamp: 'GNS 11/18/2012 12:51'!
graph
	"Answer the value of graph"

	^ graph! !

!SDNavDijkstraAlgorithm methodsFor: 'accessing' stamp: 'GNS 11/18/2012 12:51'!
graph: anObject
	"Set the value of graph"

	graph := anObject! !

!SDNavDijkstraAlgorithm methodsFor: 'accessing' stamp: 'GNS 11/18/2012 12:58'!
predecessors
	"Answer the value of predecessors"

	predecessors ifNil: [
		predecessors := Dictionary new.].
	
	^ predecessors! !

!SDNavDijkstraAlgorithm methodsFor: 'as yet unclassified' stamp: 'GNS 11/18/2012 16:17'!
shortestDistanceOf: aNode
	
	| distance |
	distance := self distances at: aNode ifAbsent: [ ^ 10000].
	^ distance.
	! !

!SDNavDijkstraAlgorithm methodsFor: 'accessing' stamp: 'GNS 11/18/2012 12:51'!
startNode
	"Answer the value of startNode"

	^ startNode! !

!SDNavDijkstraAlgorithm methodsFor: 'accessing' stamp: 'GNS 11/18/2012 12:51'!
startNode: anObject
	"Set the value of startNode"

	startNode := anObject! !

!SDNavDijkstraAlgorithm methodsFor: 'as yet unclassified' stamp: 'GNS 11/18/2012 16:08'!
unvisitedNeighborsOf: aNode

	| unvisitedNeighbors |
	
	unvisitedNeighbors := OrderedCollection new.
	
	aNode edges do: [ :anEdge |
		
		| destinationNode |
		destinationNode := anEdge to.
		
		(self visitedNodes includes: destinationNode) ifFalse: [
			unvisitedNeighbors add: destinationNode.].
		].
	
	^ unvisitedNeighbors  ! !

!SDNavDijkstraAlgorithm methodsFor: 'accessing' stamp: 'GNS 11/18/2012 13:01'!
unvisitedNodes
	"Answer the value of unvisitedNodes"

	unvisitedNodes ifNil: [
		unvisitedNodes := Set new.].
	
	^ unvisitedNodes! !

!SDNavDijkstraAlgorithm methodsFor: 'accessing' stamp: 'GNS 11/18/2012 13:01'!
visitedNodes
	"Answer the value of visitedNodes"

	visitedNodes ifNil: [
		visitedNodes := Set new.].
	
	^ visitedNodes! !

Object subclass: #SDNavGraph
	instanceVariableNames: 'nodes myCanvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SoftwareDesign-Model'!

SDNavGraph subclass: #SDNavBerlinSBahnGraph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SoftwareDesign-Model'!

!SDNavBerlinSBahnGraph methodsFor: 'initialization' stamp: 'GNS 11/20/2012 01:16'!
initialize

	super initialize.
	
	self initializeGraphStructure.! !

!SDNavBerlinSBahnGraph methodsFor: 'initialization' stamp: 'GNS 11/20/2012 01:18'!
initializeGraphStructure	
	
	| nodeA nodeB nodeC nodeD nodeE |
	
	nodeA := SDNavNode createWithName: 'NodeA'.
	nodeB := SDNavNode createWithName: 'NodeB'.   
	nodeC := SDNavNode createWithName: 'NodeC'.
	nodeD := SDNavNode createWithName: 'NodeD'.   
	nodeE := SDNavNode createWithName: 'NodeE'.
	
	nodeA addConnectionTo: nodeB withWeight: 1.
	nodeA addConnectionTo: nodeC withWeight: 1.
	nodeC addConnectionTo: nodeD withWeight: 1.
	nodeD addConnectionTo: nodeE withWeight: 1.
	nodeE addConnectionTo: nodeA withWeight: 1.   
	
	self
		addNode: nodeA;
		addNode: nodeB;
		addNode: nodeC;
		addNode: nodeD;
		addNode: nodeE.! !

!SDNavGraph methodsFor: 'accessing' stamp: 'GNS 11/20/2012 00:47'!
addNode: aNode

	self nodes add: aNode.! !

!SDNavGraph methodsFor: 'drawing' stamp: 'GNS 11/20/2012 16:49'!
drawEdges

	self nodes do: [ :aNode |
		aNode edges do: [:anEdge |
			anEdge drawYourselfOn: self myCanvas.].
		].
! !

!SDNavGraph methodsFor: 'drawing' stamp: 'GNS 11/20/2012 16:49'!
drawNodes

	self nodes do: [ :aNode |
		aNode drawYourselfOn: self myCanvas.].! !

!SDNavGraph methodsFor: 'drawing' stamp: 'GNS 11/20/2012 00:56'!
drawYourselfOn: aMorph

	
	self myCanvas: aMorph;
		drawNodes;
		drawEdges.! !

!SDNavGraph methodsFor: 'initialization' stamp: 'GNS 11/20/2012 00:39'!
initialize

	super initialize.! !

!SDNavGraph methodsFor: 'accessing' stamp: 'GNS 11/20/2012 00:56'!
myCanvas
	"Answer the value of myCanvas"

	^ myCanvas! !

!SDNavGraph methodsFor: 'accessing' stamp: 'GNS 11/20/2012 00:56'!
myCanvas: anObject
	"Set the value of myCanvas"

	myCanvas := anObject! !

!SDNavGraph methodsFor: 'accessing' stamp: 'GNS 11/18/2012 15:43'!
nodes
	
	nodes ifNil: [
		nodes := OrderedCollection new.].
	
	^ nodes! !

!SDNavGraph methodsFor: 'accessing' stamp: 'GNS 11/20/2012 00:56'!
nodes: anObject
	"Set the value of nodes"

	nodes := anObject! !

SDNavGraph subclass: #SDNavLittleSampleGraph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SoftwareDesign-Model'!

!SDNavLittleSampleGraph methodsFor: 'initialization' stamp: 'GNS 11/20/2012 00:38'!
initialize

	super initialize.
	
	self initializeGraphStructure.! !

!SDNavLittleSampleGraph methodsFor: 'initialization' stamp: 'GNS 11/20/2012 00:37'!
initializeGraphStructure
	
	| nodeA nodeB nodeC |
	
	nodeA := SDNavNode createWithName: 'A' on: 3@5.
	nodeB := SDNavNode createWithName: 'B' on: 4@2.   
	nodeC := SDNavNode createWithName: 'C' on: 3@3.
	
	nodeA addConnectionTo: nodeB withWeight: 2.
	nodeA addConnectionTo: nodeC withWeight: 5.
	nodeB addConnectionTo: nodeC withWeight: 2.
	   
	
	self
		addNode: nodeA;
		addNode: nodeB;
		addNode: nodeC.
	
	! !

Object subclass: #SDNavGraphRenderer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SoftwareDesign'!

!SDNavGraphRenderer class methodsFor: 'as yet unclassified' stamp: 'GNS 11/20/2012 00:45'!
drawEdges

	self nodes do: [ :node |
		self drawEdgesFor: node.].! !

!SDNavGraphRenderer class methodsFor: 'as yet unclassified' stamp: 'GNS 11/20/2012 00:44'!
drawEdgesFor: aNode

	aNode edges do: [ :edge |
		
		self addMorph: edge.
		
		
		edge vertices: (Array with: (aNode position) with: (edge to position)) color: Color black borderWidth: 2 borderColor: Color black.
		edge makeForwardArrow.
		edge computeBounds.
		].
! !

!SDNavGraphRenderer class methodsFor: 'as yet unclassified' stamp: 'GNS 11/20/2012 00:53'!
drawNodesOf: aGraph on: aMorph
	
	self
		drawNodes.
	
	"aMorph addMorph: aNode. 
	aNode extent: 30@30; 
		position: self position + (aNode gridPosition * 50).
		
	aGraph nodes do: [ :node |
		self drawEdgesFor: node.]."
! !

!SDNavGraphRenderer class methodsFor: 'as yet unclassified' stamp: 'GNS 11/20/2012 00:52'!
render: aGraph on: aMorph
	
	self
		drawNodes.
	
	"aMorph addMorph: aNode. 
	aNode extent: 30@30; 
		position: self position + (aNode gridPosition * 50).
		
	aGraph nodes do: [ :node |
		self drawEdgesFor: node.]."
! !

PolygonMorph subclass: #SDNavEdge
	instanceVariableNames: 'startNode endNode weights'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SoftwareDesign-Model'!

!SDNavEdge class methodsFor: 'instance creation' stamp: 'GNS 11/18/2012 12:21'!
createFrom: aSource to: aDestination withWeight: aWeight

	| edge |
	
	edge := SDNavEdge new.
	edge from: aSource.
	aSource edges add: edge.
	edge to: aDestination.
	edge weights: aWeight.		
	
	^ edge ! !

!SDNavEdge class methodsFor: 'instance creation' stamp: 'GNS 11/20/2012 16:45'!
newWithStartNode: aStartNode andEndNode: anEndNode withWeight: aWeight

	| anEdge |
	
	anEdge := SDNavEdge new.
	anEdge
		startNode: aStartNode;
		endNode: anEndNode;
		weights: aWeight.
	
	^ anEdge ! !

!SDNavEdge methodsFor: 'drawing' stamp: 'GNS 11/20/2012 16:48'!
drawYourselfOn: aCanvas

	| startNode endNode startPoint endPoint xDifference yDifference |
	
	aCanvas addMorph: self.
	
	startNode := self startNode.
	endNode := self endNode.
	
	startPoint := startNode position.
	endPoint := endNode position.
	
	xDifference := startPoint x - endPoint x.
	yDifference := startPoint y - endPoint y.
	(xDifference abs < yDifference abs ) 
		ifTrue:
			[(yDifference < 0) 
			ifTrue: 
				[startPoint := startNode bottomCenter.
				endPoint := endNode topCenter.]
			ifFalse:
				[startPoint := startNode topCenter.
				endPoint := endNode bottomCenter.]] 
		ifFalse: 
			[(xDifference < 0) 
			ifTrue: 
				[startPoint := startNode rightCenter.
				endPoint := endNode leftCenter.]
			ifFalse:
				[startPoint := startNode leftCenter.
				endPoint := endNode rightCenter.]].
	self 
		vertices: (Array 
					with: (startPoint)
					with: (endPoint))
		color: Color black
		borderWidth: 2
		borderColor: Color black;
		makeForwardArrow;
		computeBounds.! !

!SDNavEdge methodsFor: 'accessing' stamp: 'GNS 11/20/2012 16:38'!
endNode
	"Answer the value of endNode"

	^ endNode! !

!SDNavEdge methodsFor: 'accessing' stamp: 'GNS 11/20/2012 16:38'!
endNode: anObject
	"Set the value of endNode"

	endNode := anObject! !

!SDNavEdge methodsFor: 'accessing' stamp: 'GNS 11/20/2012 16:38'!
startNode
	"Answer the value of startNode"

	^ startNode! !

!SDNavEdge methodsFor: 'accessing' stamp: 'GNS 11/20/2012 16:38'!
startNode: anObject
	"Set the value of startNode"

	startNode := anObject! !

!SDNavEdge methodsFor: 'accessing' stamp: 'GNS 11/18/2012 12:20'!
weights
	"Answer the copy of the weights"

	^ weights! !

!SDNavEdge methodsFor: 'accessing' stamp: 'GNS 11/18/2012 11:52'!
weights: anObject
	"Set the value of weights"

	weights := anObject! !

SystemWindow subclass: #SDNavMainWindow
	instanceVariableNames: 'graph graphCanvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SoftwareDesign'!

!SDNavMainWindow methodsFor: 'accessing' stamp: 'GNS 11/20/2012 01:10'!
graph
	"Answer the value of graph"

	^ graph! !

!SDNavMainWindow methodsFor: 'accessing' stamp: 'GNS 11/20/2012 01:10'!
graph: anObject
	"Set the value of graph"

	graph := anObject! !

!SDNavMainWindow methodsFor: 'accessing' stamp: 'GNS 11/20/2012 01:10'!
graphCanvas
	"Answer the value of graphCanvas"

	^ graphCanvas! !

!SDNavMainWindow methodsFor: 'accessing' stamp: 'GNS 11/20/2012 01:10'!
graphCanvas: anObject
	"Set the value of graphCanvas"

	graphCanvas := anObject! !

!SDNavMainWindow methodsFor: 'initialization' stamp: 'GNS 11/20/2012 00:09'!
initialize	
	
	super initialize.
	
	self initializeAppearance.
	  
	
	
	
	
	! !

!SDNavMainWindow methodsFor: 'initialization' stamp: 'GNS 11/20/2012 01:09'!
initializeAppearance

	self setLabel: 'Navigation System';
		model: Model new;
		initializeGraphCanvas;
		initializeControlPanel.! !

!SDNavMainWindow methodsFor: 'initialization' stamp: 'GNS 11/20/2012 00:09'!
initializeControlPanel

	| controlPanel |
	
	controlPanel := Morph new.
	self addMorph: controlPanel fullFrame: (LayoutFrame fractions: (0@0 corner: 1@0) offsets: (0@0 corner: 0@50)).
	controlPanel color: Color lightGreen.
	
	self initializeSimpleMapLoadButton: controlPanel.
	 ! !

!SDNavMainWindow methodsFor: 'initialization' stamp: 'GNS 11/20/2012 01:10'!
initializeGraphCanvas

	| graphCanvas |
	
	graphCanvas := Morph new.
	self addMorph: graphCanvas fullFrame: (LayoutFrame fractions: (0@0 corner: 1@1) offsets: (0@50 corner: 0@0)).
	graphCanvas color: Color lightGray.
	
	self graphCanvas: graphCanvas.! !

!SDNavMainWindow methodsFor: 'initialization' stamp: 'GNS 11/20/2012 00:12'!
initializeSimpleMapLoadButton: aControlPanel

	| loadButton |
	
	loadButton := self makeButton: 'Load' withAction: #loadSimpleMap andState: nil onControlPanel: aControlPanel.
	
	aControlPanel addMorph: loadButton.! !

!SDNavMainWindow methodsFor: 'action-handling' stamp: 'GNS 11/20/2012 16:48'!
loadSimpleMap

	self graph: SDNavLittleSampleGraph new.
	self graph drawYourselfOn: self graphCanvas.
! !

!SDNavMainWindow methodsFor: 'initialization' stamp: 'GNS 11/20/2012 00:11'!
makeButton: aLabel withAction: anActionSymbol andState: aStateSymbol
	
	| button label |
	
	button := PluggableButtonMorph 
				on: self
				getState: aStateSymbol
				action: anActionSymbol.
				
	label := StringMorph contents: aLabel.
	label emphasis: 1.
	
	button
		label: label;
		useRoundedCorners;
		height: 30;
		onColor: (Color veryLightGray) offColor: (Color lightGray);
		borderWidth: 2;
		borderColor: #raised.		
	button label color: Color darkGray.
	
	^ button! !

!SDNavMainWindow methodsFor: 'initialization' stamp: 'GNS 11/20/2012 00:28'!
makeButton: aLabel withAction: anActionSymbol andState: aStateSymbol onControlPanel: aControlPanel
	
	| button |
	
	button := SimpleButtonMorph new.
   	button
		target: self;
		label: aLabel;
		center: 30 @ (aControlPanel height / 2);
		actionSelector: anActionSymbol.
		
	^ button! !
